import { useCallback, useContext, useEffect, useRef, useState } from 'react';
import { useInitializeEffect } from '..';
import { ClientContext } from '../../context/clientContext';
export var useQuery = function (action, initFetch) {
    if (initFetch === void 0) { initFetch = true; }
    var clientContext = useContext(ClientContext);
    var isMounted = useRef(true);
    var _a = useState(false), isError = _a[0], setError = _a[1];
    var _b = useState(true), isLoading = _b[0], setLoading = _b[1];
    var _c = useState(null), response = _c[0], setResponse = _c[1];
    useEffect(function () {
        isMounted.current = true;
        return function () {
            isMounted.current = false;
        };
    });
    var handleFetch = useCallback(function (keepLastResponse) {
        if (!isMounted.current) {
            return;
        }
        setLoading(true);
        setError(false);
        if (!keepLastResponse) {
            setResponse(null);
        }
        clientContext
            .fetch(action)
            .then(function (fetchResponse) {
            if (fetchResponse.error) {
                handleError();
            }
            else {
                handleSuccess(fetchResponse);
            }
        })
            .catch(function (error) {
            handleError();
        });
    }, [action]);
    useInitializeEffect(function () {
        if (initFetch) {
            handleFetch();
        }
    });
    var handleSuccess = function (fetchResponse) {
        if (!isMounted.current) {
            return;
        }
        setLoading(false);
        setResponse(fetchResponse.payload);
    };
    var handleError = function () {
        if (!isMounted.current) {
            return;
        }
        setError(true);
        setLoading(false);
    };
    return {
        error: isError,
        fetch: handleFetch,
        loading: isLoading,
        response: response,
    };
};
